//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//% CLIP2TXT.C - Copy the contents of the Windows Clipboard to a named file %
//% Written by James Greene Informatics Consulting, Lrrach, Germany        %
//%-------------------------------------------------------------------------%
//% Last Modified:                                                          %
//%     21.01.1999 - Release of version 1.0                                 %
//%-------------------------------------------------------------------------%
//% CLIP2TXT Version 1.0 (21. January 1999)                                 %
//% Copyright (c) 1999 James Greene Informatics Consulting                  %
//% CLIP2TXT comes with NO WARRANTY to the extent permitted by law.         %
//% You may redistribute copies of CLIP2TXT under the terms of the          %
//% GNU General Public License. For more information about these            %
//% matters, see the file named COPYING, or contact the author at           %
//% <greene@gucc.org>                                                       %
//%                                                                         %
//% Please report any bugs to <greene@gucc.org>                             %
//%                                                                         %
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <errno.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define VER_NUMBER  "1.0a"
#define VER_STRING  "Version " VER_NUMBER
#define VER_DATE    "21. January 1999"

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

typedef struct tagOPTIONS {
    int fQuiet: 1;
    int fTranslate: 1;
    int fDisplay: 1;
    int fHelp: 1;
    int fVersion: 1;
    char szName[MAX_PATH];
} OPTIONS;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  void    AnyKey(void) {
    register int ri;
    char szMsg[32];

    strcpy(szMsg, "(press any key to continue)");
    
    fprintf(stderr, szMsg);
    fflush(stderr);
    
    if (!getch())
        getch();
        
    for (ri=strlen(szMsg) ; ri > 0; ri--) 
        fprintf(stderr, "\b \b");
    fflush(stderr);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static void Version(void) {
    fprintf(stderr, "CLIP2TXT Version %s (%s at %.5s)\n", VER_NUMBER, VER_DATE, __TIME__);
    fprintf(stderr, "Copyright (c) 1999 James Greene Informatics Consulting\n");
    fprintf(stderr, "CLIP2TXT comes with NO WARRANTY to the extent permitted by law.\n");
    fprintf(stderr, "You may redistribute copies of CLIP2TXT under the terms of the\n");
    fprintf(stderr, "GNU General Public License. For more information about these\n");
    fprintf(stderr, "matters, see the file named COPYING.txt, or contact the author\n");
    fprintf(stderr, "at <greene@gucc.org>\n");
    fprintf(stderr, "\nPlease report any bugs to <greene@gucc.org>\n\n");
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  void    Usage(char *exe) {
    fprintf(stderr, "Usage: %s [options] filename [options]\n", exe);
    fprintf(stderr, "Options:  ( /option or --option are also accepted ) \n");
    fprintf(stderr, "  -d[isplay]   = Display the clipboard contents on the screen.\n");
    fprintf(stderr, "  -h[elp]      = Display this usage information.\n");
    fprintf(stderr, "  -n[ologo]    = Don't display the Copyright banner.\n");
    fprintf(stderr, "  -q[uiet]     = Same as -nologo.\n");
    fprintf(stderr, "  -s[ilent]    = Same as -nologo.\n");
    fprintf(stderr, "  -t[ranslate] = Convert from Windows ANSI to MS-DOS ASCII codes.\n");
    fprintf(stderr, "  -?           = Same as -help.\n");
    fprintf(stderr, "  The following options are all interpreted the same:\n");
    fprintf(stderr, "     -d, /d, -DISP, /Disp, --DISPLAY, -dIsP, ... you get the idea!\n\n");
    fprintf(stderr, "  If no filename is specified, the clipboard contents will be displayed on\n");
    fprintf(stderr, "  the screen, so CLIP2TXT > FOO.TXT is equivalent to CLIP2TXT -T FOO.TXT\n\n");

    AnyKey();
    
    fprintf(stderr, "Examples:\n");
    fprintf(stderr, "  %s FOO.TXT\n", exe);
    fprintf(stderr, "  Copies the Windows Clipboard contents to the file FOO.TXT in the current\n");
    fprintf(stderr, "  directory.\n");
    fprintf(stderr, "  %s -t \"C:\\My Documents\\Long File Name.txt\"\n", exe);
    fprintf(stderr, "  Copies the Clipboard to the specified file. Filenames containing spaces must\n");
    fprintf(stderr, "  be \"enclosed in double quotes.\"\n\n");

    Version();
}
    
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

static  BOOL ParseCommandLine(char *argv[], OPTIONS *opt) {
    char *pgm, *arg;
    register int ri;

    memset(opt, 0, sizeof(OPTIONS));

    pgm = *argv;
    
    while (arg = *++argv) {
        if ((*arg == '/') || (*arg == '-')) {   // flag
            ri = strlen(++arg);
            if (*arg == '-')
                ri = strlen(++arg);
          
            if (_strnicmp(arg, "TRANSLATE", ri) == 0) 
                opt->fTranslate = TRUE;
            else if (_strnicmp(arg, "DISPLAY", ri) == 0)
                opt->fDisplay = TRUE;
            else if ((_strnicmp(arg, "NOLOGO", ri) == 0) ||
                      (_strnicmp(arg, "QUIET", ri) == 0) ||
                      (_strnicmp(arg, "SILENT", ri) == 0))
                opt->fQuiet = TRUE;
            else if ((_strnicmp(arg, "HELP", ri) == 0) ||
                      (_strnicmp(arg, "?", ri) == 0))
                opt->fHelp = TRUE;
            else if ((_strnicmp(arg, "VERSION", ri) == 0))
                opt->fVersion = TRUE;
            else
                fprintf(stderr, "CLIP2TXT: Option '%s' not recognised\n", --arg);
        } else {
            if (!*(opt->szName))
                strcpy(opt->szName, arg);
            else
                fprintf(stderr, "CLIP2TXT: Too many parameters: '%s' ignored\n", arg);
        }
    }

    if (!opt->fQuiet) {
        fprintf(stderr, "CLIP2TXT: Copy the Windows Clipboard contents to a File     %14s\n", VER_STRING);
        fprintf(stderr, "Copyright (c) 1999 James Greene Informatics Consulting   <greene@gucc.org>\n");
        fprintf(stderr, "                   http://gucc.org/greene/consult\n\n");
    }

    // if no filename specified, then just display the text to the screen
    if (!*(opt->szName)) {
        opt->fTranslate = TRUE;
        opt->fDisplay = TRUE;
    }

    if (opt->fVersion) 
        Version();
        
    if (opt->fHelp) {
        Usage(pgm);
        return FALSE;
    }

    return TRUE;
}
        
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
static  BOOL    ClipCopy(OPTIONS *opt) {
    HANDLE hd;
    LPSTR lp;
    register int ri;
    FILE *fp;

    if (!OpenClipboard(NULL)) {
        fprintf(stderr, "CLIP2TXT: Unable to open the Windows clipboard!\n");
        return FALSE;
    }

    if (!(hd = GetClipboardData(opt->fTranslate ? CF_OEMTEXT : CF_TEXT))) {
        fprintf(stderr, "CLIP2TXT: There is no text in the Windows clipboard!\n");
        CloseClipboard();
        return FALSE;
    }

    if (*opt->szName) {
        if ((fp = fopen(opt->szName, "wb")) == NULL) {
            fprintf(stderr, "CLIP2TXT: Unable to open file %s: %s\n",
                    opt->szName, strerror(errno));
            CloseClipboard();
            return FALSE;
        }
    } else
        fp = NULL;

    if (lp = GlobalLock(hd)) {
        ri = lstrlen(lp);

        if (fp) {
            fputs(lp, fp);
            fclose(fp);
        }

        if (opt->fDisplay) 
            fputs(lp, stdout);
            
         GlobalUnlock(hd);
    } else {    // this should NEVER happen!
        fprintf(stderr, "CLIP2TXT: Unable to lock the clipboard memory!\n");
        if (fp)
            fclose(fp);
        return FALSE;
    }
        
    CloseClipboard();

    return TRUE;
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
int main(int argc, char *argv[]) {
    OPTIONS opt;

    return ParseCommandLine(argv, &opt) && ClipCopy(&opt) ? 1 : 0;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

